import { IDisposable } from '@lumino/disposable';
import { ISignal } from '@lumino/signaling';
import { ISessionContext } from '@jupyterlab/apputils';
import { IDocumentManager } from '@jupyterlab/docmanager';
import { NotebookPanel } from '@jupyterlab/notebook';
import { KernelMessage } from '@jupyterlab/services';
import { IQueryResult, IReport } from '../linter/interfaces';
import { Languages } from '../linter/languages';
import { Config } from './config';
import { ExperimentManager } from './experimentmanager';
import { OptionsManager } from './optionsmanager';
import { ErrorHandler } from './errorhandler';
export interface IJulynterKernelUpdate {
    status: string;
    kernelName?: string;
    result?: IQueryResult;
}
export declare class NotebookHandler implements IDisposable {
    private _language;
    private _kernelRestarted;
    private _disposed;
    private _inspected;
    private _isDisposed;
    private _ready;
    private _panelId;
    private _nbPanel;
    private _attempts;
    private _session;
    private _docManager;
    private _update;
    private _experimentManager;
    private _eh;
    private _reportedStart;
    options: OptionsManager;
    update: IQueryResult | null;
    hasKernel: boolean;
    constructor(docManager: IDocumentManager, session: ISessionContext, nbPanel: NotebookPanel, config: Config, em: ExperimentManager, eh: ErrorHandler, update: () => void);
    getKernelLanguage(): Promise<Languages.LanguageModel>;
    configureHandler(language: Languages.LanguageModel): void;
    disconnectHandler(): void;
    connectHandler(): void;
    get name(): string;
    get id(): string;
    get nbPanel(): NotebookPanel;
    get experimentManager(): ExperimentManager;
    /**
     * A signal emitted when the handler is disposed.
     */
    get disposed(): ISignal<NotebookHandler, void>;
    get isDisposed(): boolean;
    get ready(): Promise<void>;
    /**
     * A signal emitted when an inspector value is generated.
     */
    get inspected(): ISignal<NotebookHandler, IJulynterKernelUpdate>;
    /**
     * Disposes the kernel connector.
     */
    dispose(): void;
    /**
     * Lint notebook
     */
    lint(): IReport[];
    private _createPromise;
    private executeLanguageCommand;
    /**
     * Send a query command to the kernel
     */
    performQuery(): void;
    /**
     * Send message to kernel add a module
     */
    addModule(module: string): void;
    /**
     * Initializes the kernel by running the set up script located at _initScriptPath.
     */
    private _initOnKernel;
    private _handleQueryResponse;
    private _queryCall;
    /**
     * Handle kernel signals.
     */
    protected onQueryUpdate(sender: NotebookHandler, update: IJulynterKernelUpdate): void;
    /**
     * Handle disposed signals.
     */
    protected onSourceDisposed(sender: NotebookHandler, args: void): void;
    /**
     * Executes the given request on the kernel associated with the connector.
     */
    sendToKernel(content: KernelMessage.IExecuteRequestMsg['content'], ioCallback: ((content: KernelMessage.IExecuteRequestMsg['content'], msg: KernelMessage.IIOPubMessage) => any) | null): Promise<void>;
}
