import React from 'react';
import ReactDOM from 'react-dom';
import { Widget } from '@lumino/widgets';
import { Dialog, MainAreaWidget, ReactWidget, showDialog, showErrorMessage, Toolbar, ToolbarButtonComponent, } from '@jupyterlab/apputils';
import { HTMLSelect } from '@jupyterlab/ui-components';
import { ERRORS, ERROR_TYPES_MAP } from '../../linter/reports';
import { ErrorTypeKeys, ReportIds, ViewModes, } from '../../linter/interfaces';
const MODES = {
    list: 'View as list',
    cell: 'Group by cell',
    type: 'Group by lint type',
};
class ReportGroup extends React.Component {
    /* eslint @typescript-eslint/no-unused-vars: 0 */
    constructor(props) {
        super(props);
        this.state = { collapsed: !this.props.current.types[this.props.group] };
    }
    clickItem(key) {
        return () => {
            try {
                this.props.current.reports[key] = !this.props.current.reports[key];
                this.forceUpdate();
            }
            catch (error) {
                throw this.props.errorHandler.report(error, 'ReportGroup:clickItem', [
                    key,
                ]);
            }
        };
    }
    changeItem() {
        return;
    }
    changeGroup(event) {
        try {
            const updateLater = this.props.current.types[this.props.group] !== this.state.collapsed;
            this.props.current.types[this.props.group] = event.target.checked;
            if (updateLater) {
                this.setState({
                    collapsed: !event.target.checked,
                });
            }
            this.forceUpdate();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ReportGroup:changeGroup', [
                this.props.group,
                event.target.checked,
            ]);
        }
    }
    collapseGroup() {
        try {
            this.setState({
                collapsed: !this.state.collapsed,
            });
            this.forceUpdate();
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ReportGroup:collapseGroup', [
                this.props.group,
            ]);
        }
    }
    render() {
        try {
            let twistButton = null;
            const items = this.props.items.map((el) => {
                const key = 'config-report-' + el.key;
                return (React.createElement("tr", { key: key, onClick: this.clickItem(el.key) },
                    React.createElement("td", null,
                        React.createElement("input", { type: "checkbox", onChange: this.changeItem, checked: this.props.current.reports[el.key] })),
                    React.createElement("td", null, el.key),
                    React.createElement("td", null, el.item.label('<T1>', '<T2>', '<T3>', '<T4>', '<T5>')),
                    React.createElement("td", null, el.item.suggestion)));
            });
            let notice = null;
            if (!this.props.current.types[this.props.group]) {
                notice = (React.createElement("span", null, "This category is disabled. The individual options in the following table will have no effect."));
            }
            let children = (React.createElement("div", null,
                notice,
                React.createElement("table", null,
                    React.createElement("thead", null,
                        React.createElement("tr", null,
                            React.createElement("th", null, "Use"),
                            React.createElement("th", null, "Code"),
                            React.createElement("th", null, "Template"),
                            React.createElement("th", null, "Suggestion"))),
                    React.createElement("tbody", null, items))));
            if (this.state.collapsed) {
                children = null;
                twistButton = (React.createElement("div", { className: "julynter-collapse-button" },
                    React.createElement("div", { className: "julynter-twist-placeholder" }, "placeholder"),
                    React.createElement("div", { className: "julynter-rightarrow-img julynter-arrow-img" })));
            }
            else {
                twistButton = (React.createElement("div", { className: "julynter-collapse-button" },
                    React.createElement("div", { className: "julynter-twist-placeholder" }, "placeholder"),
                    React.createElement("div", { className: "julynter-downarrow-img julynter-arrow-img" })));
            }
            const errorType = ERROR_TYPES_MAP[this.props.group];
            return (React.createElement("div", { className: "julynter-config-group" },
                React.createElement("h2", null,
                    React.createElement("div", { className: "julynter-config-group-head", onClick: this.collapseGroup.bind(this) },
                        twistButton,
                        React.createElement("span", null, errorType.label)),
                    React.createElement("input", { type: "checkbox", onChange: this.changeGroup.bind(this), checked: this.props.current.types[this.props.group] })),
                children));
        }
        catch (error) {
            throw this.props.errorHandler.report(error, 'ReportGroup:render', [
                this.props.group,
            ]);
        }
    }
}
export class JulynterConfigContent extends Widget {
    constructor(checks, eh) {
        super();
        this._eh = eh;
        this.addClass('julynter-config');
        this._key = 0;
        this._current = checks;
        this.display();
    }
    externalUpdate() {
        try {
            this._key += 1;
            this.display();
        }
        catch (error) {
            throw this._eh.report(error, 'JulynterConfigContent:externalUpdade', []);
        }
    }
    selectMode(event) {
        try {
            this._current.mode = event.currentTarget.value;
        }
        catch (error) {
            throw this._eh.report(error, 'JulynterConfigContent:selectMode', [
                event.currentTarget.value,
            ]);
        }
    }
    writeRequirements(event) {
        try {
            this._current.requirements = event.currentTarget.value;
        }
        catch (error) {
            throw this._eh.report(error, 'JulynterConfigContent:writeRequirements', [
                event.currentTarget.value,
            ]);
        }
    }
    display() {
        try {
            let renderedJSX = null;
            const groups = {};
            for (const key of ReportIds) {
                const error = ERRORS[key];
                if (error.type in groups) {
                    groups[error.type].push({
                        key: key,
                        item: error,
                    });
                }
                else {
                    groups[error.type] = [
                        {
                            key: key,
                            item: error,
                        },
                    ];
                }
            }
            const reportGroups = ErrorTypeKeys.map((el) => {
                const key = 'config-' + el + '-' + this._key;
                return (React.createElement(ReportGroup, { current: this._current, key: key, group: el, items: groups[el], errorHandler: this._eh }));
            });
            renderedJSX = (React.createElement("div", { className: "julynter-config-inner" },
                React.createElement("h1", null, " Configure Julynter "),
                React.createElement("div", { className: "julynter-config-notice" }, "Do not forget to click the \"Save\" button when you are done"),
                React.createElement("label", { key: 'mode-' + this._key },
                    "Mode:",
                    React.createElement(HTMLSelect, { className: "julynter-config-select", defaultValue: this._current.mode, onChange: this.selectMode.bind(this), options: ViewModes.map((el) => {
                            return { label: MODES[el], value: el };
                        }) })),
                React.createElement("label", null,
                    "Requirements file:",
                    React.createElement("div", { className: "julynter-config-input", key: 'req-' + this._key },
                        React.createElement("input", { type: "text", onChange: this.writeRequirements.bind(this), defaultValue: this._current.requirements }))),
                reportGroups));
            ReactDOM.render(renderedJSX, this.node);
        }
        catch (error) {
            throw this._eh.report(error, 'JulynterConfigContent:display', []);
        }
    }
}
export class ContextSwitcher extends ReactWidget {
    constructor(tracker, selected, content, eh) {
        super();
        this._eh = eh;
        this._tracker = tracker;
        this._content = content;
        this.selected = selected;
        this.update();
        tracker.widgetAdded.connect((sender, nbPanel) => {
            this.update();
        });
    }
    getSelected() {
        try {
            let result = this.selected;
            this._tracker.forEach((notebook) => {
                if (notebook.id === this.selected) {
                    result = notebook;
                }
            });
            return result;
        }
        catch (error) {
            throw this._eh.report(error, 'ContextSwitcher:getSelected', []);
        }
    }
    /**
     * Handle `change` events for the HTMLSelect component.
     */
    handleChange(event) {
        try {
            this.selected = event.target.value;
        }
        catch (error) {
            throw this._eh.report(error, 'ContextSwitcher:handleChange', [
                event.target.value,
            ]);
        }
    }
    /**
     * Handle `keydown` events for the HTMLSelect component.
     */
    handleKeyDown(event) {
        try {
            if (event.keyCode === 13) {
                this._content.activate();
            }
        }
        catch (error) {
            throw this._eh.report(error, 'ContextSwitcher:handleKeyDown', [
                event.keyCode,
            ]);
        }
    }
    render() {
        try {
            const value = this.selected;
            let i = 0;
            const result = [];
            this._tracker.forEach((notebook) => {
                const key = 'notebook-save-' + i++;
                result.push(React.createElement("option", { key: key, value: notebook.id },
                    notebook.title.label,
                    " Settings"));
            });
            return (React.createElement(HTMLSelect, { onChange: this.handleChange.bind(this), onKeyDown: this.handleKeyDown.bind(this), defaultValue: value, "aria-label": "Cell type" },
                React.createElement("option", { value: "user" }, "User Settings"),
                React.createElement("option", { value: "project" }, "Project Settings"),
                result));
        }
        catch (error) {
            throw this._eh.report(error, 'ContextSwitcher:render', []);
        }
    }
}
export class SaveButton extends ReactWidget {
    constructor(switcher, handlers, config, checks, eh) {
        super();
        this._eh = eh;
        this._switcher = switcher;
        this._checks = checks;
        this._handlers = handlers;
        this._config = config;
        this._props = {
            className: 'julynter-save',
            tooltip: 'Save configuration',
            label: 'Save',
            onClick: this.onClick.bind(this),
        };
        this.addClass('jp-ToolbarButton');
    }
    onClick() {
        try {
            const selected = this._switcher.selected;
            if (selected === 'user') {
                this._config
                    .saveUser(this._checks)
                    .then(() => {
                    showDialog({
                        title: 'Success',
                        body: (React.createElement("div", null,
                            React.createElement("div", null, "Stored settings for user."),
                            React.createElement("div", null, "It will not change existing notebooks that have their own settings."))),
                        buttons: [Dialog.okButton({ label: 'Ok' })],
                    });
                })
                    .catch((reason) => {
                    showErrorMessage('Failed', reason);
                });
            }
            else if (selected === 'project') {
                this._config
                    .saveProject(this._checks)
                    .then(() => {
                    showDialog({
                        title: 'Success',
                        body: (React.createElement("div", null,
                            React.createElement("div", null, "Stored settings for project."),
                            React.createElement("div", null, "It will not change existing notebooks that have their own settings."))),
                        buttons: [Dialog.okButton({ label: 'Ok' })],
                    });
                })
                    .catch((reason) => {
                    showErrorMessage('Failed', reason);
                });
            }
            else if (selected in this._handlers) {
                this._handlers[selected].then((handler) => {
                    handler.options.initializeOptions(this._checks);
                    handler.options.saveOptions();
                    showDialog({
                        title: 'Success',
                        body: (React.createElement("div", null,
                            React.createElement("div", null,
                                "Stored settings for ",
                                handler.name,
                                "."))),
                        buttons: [Dialog.okButton({ label: 'Ok' })],
                    });
                });
            }
            else {
                showErrorMessage('Failed', 'Target not found!');
            }
        }
        catch (error) {
            throw this._eh.report(error, 'SaveButton:onClick', []);
        }
    }
    render() {
        return React.createElement(ToolbarButtonComponent, Object.assign({}, this._props));
    }
}
export class LoadButton extends ReactWidget {
    constructor(switcher, handlers, config, checks, content, eh) {
        super();
        this._eh = eh;
        this._switcher = switcher;
        this._checks = checks;
        this._handlers = handlers;
        this._config = config;
        this._content = content;
        this._props = {
            className: 'julynter-load',
            tooltip: 'Load configuration',
            label: 'Load',
            onClick: this.onClick.bind(this),
        };
        this.addClass('jp-ToolbarButton');
    }
    onClick() {
        try {
            const selected = this._switcher.selected;
            if (selected === 'user') {
                this._config
                    .loadUser()
                    .then((config) => {
                    this._config.merge(this._checks, config);
                    this._content.externalUpdate();
                })
                    .catch((reason) => {
                    showErrorMessage('Failed', reason);
                });
            }
            else if (selected === 'project') {
                this._config
                    .loadProject()
                    .then((config) => {
                    this._config.merge(this._checks, config);
                    this._content.externalUpdate();
                })
                    .catch((reason) => {
                    showErrorMessage('Failed', reason);
                });
            }
            else if (selected in this._handlers) {
                this._handlers[selected].then((handler) => {
                    handler.options.reloadOptions();
                    this._config.merge(this._checks, handler.options.checks);
                    this._content.externalUpdate();
                });
            }
            else {
                showErrorMessage('Failed', 'Target not found!');
            }
        }
        catch (error) {
            throw this._eh.report(error, 'LoadButton:onClick', []);
        }
    }
    render() {
        return React.createElement(ToolbarButtonComponent, Object.assign({}, this._props));
    }
}
export class ResetButton extends ReactWidget {
    constructor(config, checks, content, eh) {
        super();
        this._checks = checks;
        this._config = config;
        this._content = content;
        this._eh = eh;
        this._props = {
            className: 'julynter-reset',
            tooltip: 'Reset configuration',
            label: 'Reset',
            onClick: this.onClick.bind(this),
        };
        this.addClass('jp-ToolbarButton');
    }
    onClick() {
        try {
            this._config.merge(this._checks, this._config.createDefault());
            this._content.externalUpdate();
        }
        catch (error) {
            throw this._eh.report(error, 'ResetButton:onClick', []);
        }
    }
    render() {
        return React.createElement(ToolbarButtonComponent, Object.assign({}, this._props));
    }
}
export function createJulynterConfigWidget(tracker, handlers, config, notebook, eh) {
    try {
        const checks = Object.assign({}, notebook.options.checks);
        checks.reports = Object.assign({}, checks.reports);
        const content = new JulynterConfigContent(checks, eh);
        const saveSwitcher = new ContextSwitcher(tracker, notebook.id, content, eh);
        const saveButton = new SaveButton(saveSwitcher, handlers, config, checks, eh);
        const loadButton = new LoadButton(saveSwitcher, handlers, config, checks, content, eh);
        const resetButton = new ResetButton(config, checks, content, eh);
        const toolbar = new Toolbar();
        toolbar.addItem('julynter-reset', resetButton);
        toolbar.addItem('julynter-context', saveSwitcher);
        toolbar.addItem('julynter-load', loadButton);
        toolbar.addItem('julynter-save', saveButton);
        const conf = new MainAreaWidget({ content, toolbar });
        conf.title.iconClass = 'julynter-main-icon';
        conf.title.caption = 'Julynter Config';
        conf.title.label = 'Julynter Config';
        conf.title.closable = true;
        conf.id = 'julynter-config';
        return conf;
    }
    catch (error) {
        throw eh.report(error, 'createJulynterConfigWidget', []);
    }
}
